.Dd Jul 30, 2011
.Dt drood 8
.Sh NAME
.Nm Drood
.Nd Minimal KQueue-Based HTTP/1.1 Server
.Sh SYNOPSIS
.Nm drood -r Ao server-root Ac Oo -m Ao instances Ac -i Ao interface Ac \
-p Ao port Ac -e Ao root-resource Ac -d Ao cgi-dir Ac \
-t Ao read-timeout Ac -w Ao write-timeout Ac -u Ao user Ac -g Ao group Ac \
-l Ao logfile Ac -q Ao backlog Ac -s Ao host|port|suffix Ac -x Oc
.Sh DESCRIPTION
Drood is a small and fast, minimal web server.  It is configured with
command-line options.
.Pp
.Ss DROOD SUPPORTS:
.Pp
.Bl -bullet
.It
IPv4 and IPv6 connections.
.It
HEAD, GET, and POST request methods.
.It
HTTP/1.1 persistent connections.
.It
Name-based virtual hosting.
.It
CGI and SCGI.
.It
Gzip content-encoding.
.It
Requests for single byte ranges.
.It
Request logging instead of transaction logging.
.It 
Graceful stops and restarts.
.El
.Pp
.Ss DROOD DOES NOT SUPPORT:
.Bl -bullet
.It
TLS/SSL.
.It
Access authentication.
.It
Content negotiation beyond recognizing the
"Accept-Encoding", "If-Modified-Since", "If-Unmodified-Since", and
"Range" request headers.  The only accept-encoding supported is gzip.
.It
Any response headers useful for caching server
responses beyond the "Date" and "Last-Modified" headers.
.It
generating directory listings for clients.
.It
The ~user notation to provide access to resources in
user directories, but this may be simulated with symbolic links.
.El
.Ss SYSTEM CONFIGURATION
Drood is a kqueue(2)-based multiplexer.  By default, FreeBSD limits each
process to having a maximum of 64 simultaneously open descriptors.  This
places a limit on the number of connections drood may multiplex.  The
sysctl settings below, allow for a generous number of connections.  These
values can be set at the command-line with the "sysctl" utility.  It is
recommended that they be enabled permanently by placing these lines in
/etc/sysctl.conf on the host machine.
.Bd -literal -offset left
kern.maxfiles=16384
kern.maxfilesperproc=16384
kern.kq_calloutmax=65536
kern.ipc.nmbclusters=25600
kern.ipc.maxsockets=25600
.Ed
.Pp
Drood will attempt to set the accf_http accept filter on the listening
socket, to increase performance.  For this to succeed, you will need to
load the module before starting drood with:
.Bd -literal -offset left
kldload accf_http
.Ed
.Pp
To load the module automatically at system boot-up, add the following line
to /boot/loader.conf:
.Bd -literal -offset left
accf_http_load="YES"
.Ed
.Ss DROOD CONFIGURATION
Drood writes its pid into /var/run/drood.pid, if it can (ie., it is started
as root).  It may be stopped with a SIGTERM and restarted with a SIGHUP.
.Pp
A rc.d script is provided and installed in /usr/local/etc/rc.d/.  Add the
following lines to /etc/rc.conf to start drood on system boot-up.  Replace
the items in brackets with values appropriate for your system.  These are
the minimal set of options one should start with.  These and all of the
other available options are described in full at the end of this manual
page.
.Bd -literal -offset left
drood_enable="YES"
drood_flags="-r <server-root> -u <user> -g <group>"
.Ed
.Pp
Once your system is configured you may start, stop, or restart drood, or
determine if it is running with the following commands:
.Bd -literal -offset left
/usr/local/etc/rc.d/drood start
/usr/local/etc/rc.d/drood stop
/usr/local/etc/rc.d/drood restart
/usr/local/etc/rc.d/drood status
.Ed
.Pp
If you do not want drood started on system start, then set
.Bd -literal -offset left
drood_enable="NO"
.Ed
.Pp
and use the following commands:
.Bd -literal -offset left
/usr/local/etc/rc.d/drood forcestart
/usr/local/etc/rc.d/drood forcestop
/usr/local/etc/rc.d/drood forcerestart
/usr/local/etc/rc.d/drood forcestatus
.Ed
.Pp
.Ss GZIP CONTENT ENCODING
To enable gzip content-encoding for static resources, the webmaster must
manually gzip the resources he or she would like to make available in that
encoding.  The original resource should not be replaced by the gzipped
version, but should also be available in the same directory for clients
which cannot handle the encoding.  The gzipped version must have the ".gz"
filename suffix.  If a client indicates it is acceptable for a requested
resource to be gzipped, then drood will attempt to send the gzipped version
of the resource.  If this attempt fails, drood will attempt to send the
non-compressed version of the requested resource.
.Pp
Note that drood does not compare the modification times of the uncompressed
and compressed resources.  It will always send the compressed resource if
it exists and the client wants it.  One can keep Makefiles in one's html
directories to compress all the resources whose compressed versions are out
of date, with gzip -k.  As long as the Makefiles are not readable by the
user or group drood is running as, client requests for them will fail.
.Pp
Note that when sending responses to requests with Range headers, Drood
will always send the requested byte ranges from the uncompressed resource.
Gzip content encoding is only used for responses containing entire static
resources.
.Ss MAPPING FILENAME SUFFIXES TO MIME TYPES
These are the mappings from filename suffixes to MIME types which drood
recognizes out-of-the-box.  They are listed in types.tab in
/usr/local/etc/drood/.  Any file which cannot be identified by these
mappings is identified by the server as "text/plain; charset=utf-8".  To
add a mapping, simply add a line to types.tab.  Each line must consist of
the filename suffix to be mapped, followed by one or more tab characters,
followed by the MIME type, followed by a newline.  Blank lines are ignored,
after the first line.  The first line must not be blank.  If drood finds
empty fields in types.tab it will refuse to run.  If you modify types.tab,
you must restart drood for the changes to take effect.
.Bl -column -offset left "jpeg" "application/x-shockwave-flash"
.It Sy Suffix Ta Sy MIME Type
.It html Ta text/html; charset=utf-8
.It htm  Ta text/html; charset=utf-8
.It xml  Ta text/xml; charset-utf-8
.It txt  Ta text/plain; charset=utf-8
.It css  Ta text/css
.It ico  Ta image/vnd.microsoft.icon
.It png  Ta image/png
.It jpg  Ta image/jpeg
.It jpeg Ta image/jpeg
.It tiff Ta image/tiff
.It tif  Ta image/tiff
.It gif  Ta image/gif
.It svg  Ta image/svg+xml
.It tgz  Ta application/x-compressed
.It zip  Ta application/zip
.It ps   Ta application/postscript
.It eps  Ta application/postscript
.It gz   Ta application/x-gzip
.It tgz  Ta application/x-compressed
.It pdf  Ta application/pdf
.It swf  Ta application/x-shockwave-flash
.It js   Ta application/x-javascript
.It mov  Ta video/quicktime
.It qt   Ta video/quicktime
.It mpg  Ta video/mpeg
.It mp2  Ta video/mpeg
.It mpa  Ta video/mpeg
.It mpe  Ta video/mpeg
.It avi  Ta video/x-msvideo
.It asf  Ta video/x-ms-asf
.It asx  Ta video/x-ms-asf
.It asr  Ta video/x-ms-asf
.It flv  Ta video/x-flv
.It ra   Ta audio/x-pn-realaudio
.It ram  Ta audio/x-pn-realaudio
.It wav  Ta audio/x-wav
.It mp3  Ta audio/mpeg
.El
.Pp
.Ss COMMAND-LINE OPTIONS
The following options are recognized.  Only the -r option is required.
.Bl -tag -width "-r"
.It -r
The -r option is mandatory and specifies the server root directory, which
drood will make its current working directory.
.Pp
The resources served-up by drood are determined by the "Host" header of
requests.  For every hostname you wish to be recognized by the server, an
identically-named subdirectory of the server root directory must be
created.  The static files served up by each virtual host are placed into
the identically-named subdirectory.  Any or all of the subdirectories may
be hard or symbolic links.  One subdirectory must be named "default".  This
is the directory which will be used to fulfill static requests when no
"Host" header is supplied by a client (eg., HTTP/1.0 requests).  The
"default" directory can and should be a link.  If you would like a
directory to be used for more than one hostname, you must create links for
the extra hostnames.
.Pp
An example root directory specifying entries for one hostname is below.
Only the first entry is an actual virtual host subdirectory.
.Pp
.Bd -literal -offset left
mammothcheese.ca          -> actual subdirectory (IPv4)
www.mammothcheese.ca      -> link to above (IPv4)
default                   -> link (IPv4/IPv6 HTTP/1.0)
.Ed
.Pp
If you wish to accept connections specified by IP address, you must
include links for valid addresses:
.Bd -literal -offset left
64.15.138.206             -> link (IPv4 Decimal)
127.0.0.1                 -> link (IPv4 Loopback)
.Ed
.Pp
The following three entries specify valid, but rarely used forms of IPv4
addresses, which could be included for completeness sake:
.Bd -literal -offset left
0100.017.0212.0316        -> link (IPv4 Octal)
0x40.0x0f.0x8a.0xce       -> link (IPv4 Hexadecimal)
1074760398                -> link (IPv4 IP Number)
.Ed
.Pp
If you wish to accept IPv6 connections, you will need to add entries where
the hostname is delimited with square brackets per RFC3986:  
.Bd -literal -offset left
[mammothcheese.ca]        -> link for IPv6 connections
[www.mammothcheese.ca]    -> link for IPv6 connections
[::1]                     -> link (IPv6 Loopback)
.Ed
.Pp
If you cause drood to listen on a port other than port 80, that port number
will appear in client "Host" headers.  You must adjust your entries
accordingly:  
.Bd -literal -offset left
mammothcheese.ca:8080     -> actual subdirectory (IPv4)
www.mammothcheese.ca:8080 -> link to above
(etc)
.Ed
.Pp
If the -d option is present, requests whose "Host" headers reference the
directory specified by that option, will be rejected.  This prevents
clients from requesting files in the (S)CGI directory hierarchy.
.Pp
.It -e
The root resource of all virtual hosts defaults to a static resource named
"index.html", which itself may be a symbolic link.  This may be changed
with the -e option.  Note this changes the name of the root resource of all
virtual hosts served-up by drood.  The value specified must not begin with
a virgule "/".
.Pp
If a request is received for a directory, that request is interpreted as a
request for the root resource residing in that directory.
.Pp
You may, with the -e option, specify a dynamic resource as the root
resource.  If you do, you must be sure to include the -d option to tell the
server where (S)CGI programs live.  Eg.,
.Bd -literal -offset left
drood -r /usr/local/www/drood -d cgi-bin -e cgi-bin/index.cgi
.Ed
.It -d
The -d option, if present, specifies the subdirectory of the server root
directory which contains CGI and/or SCGI resources.  The value given to
this option cannot contain the directory separator character "/" anywhere
in it, nor can it be ".." or "." by itself.  The directory specified must
be one level below the server root directory.  The directory must be
readable and searchable for the user or group specified with the -u and -g
options.  The directory may contain any subdirectory structure the user
wishes to create in it.  Only executables in the (S)CGI directory, or its
subdirectories, can be run.  Drood refuses to execute symbolic links.
.Pp
All virtual hosts share the (S)CGI directory.  If (S)CGI programs wish to
know what hostname they have been invoked for, they must examine the
SERVER_NAME environment variable.  If no "Host" header was supplied with
the request, then SERVER_NAME will be set to the string "default".
.Pp
Requests for dynamic resources must include the dynamic path in their URIs,
because this is how drood differentiates static requests from dynamic
requests.  Eg., if -d is "cgi-bin", then a request to run foobar.cgi would
look like this:
.Bd -literal -offset left
http://host.domain/cgi-bin/foobar.cgi
.Ed
.Pp
Note that if you have a (S)CGI directory and you subsequently decide to run
drood without the -d option, that directory will be treated as a virtual
host subdirectory.  You should at least change the (S)CGI directory's
ownership and/or permissions to make it inaccessible by drood.
.Pp
(S)CGI programs need only precede their response bodies with a
"Content-Type" header and a blank line.  Drood ignores Status header lines
from (S)CGI programs.  If a "Location" header is provided by the (S)CGI
program, then drood will create a "302 Found" status line to send to the
client.  Otherwise, drood will compose a "200 OK" status line.  This means
errors must be reported to the client using HTML responses.
.Pp
It is crucial that CGI programs invoked with the POST method read from
stdin exactly the number of characters specified by the CONTENT-LENGTH
environment variable.  Reading too much or too little will prevent drood
from reading the next request over a persistent connection, and it will
then drop the connection.  For GET requests, the stdin of CGI programs will
be connected to /dev/null.
.Pp
The stderr of CGI programs is connected to /dev/null.  For testing purposes
run drood with -x option and the CGI stderr will be connected to the
terminal from which drood is running.  For production use, CGI programs
must send diagnostic output to files, or use syslog(3).
.Pp
The following environment variables are set for (S)CGI programs:
.Pp
REMOTE_ADDR, REQUEST_METHOD, SCRIPT_NAME, QUERY_STRING, HTTP_USER_AGENT,
HTTP_COOKIE, SERVER_SOFTWARE, SERVER_NAME, SERVER_PORT, GATEWAY_INTERFACE,
CONTENT_TYPE, SCGI, GZIP, and CONTENT_LENGTH.
.Pp
SCGI is set to 1 for SCGI resources, and to 0 for plain CGI.  If the client
request included an Accept-Encoding header indicating the client can
receive gzipped data, GZIP is set to 1.  Otherwise it is set to 0.
.Pp
Drood cannot recognize virtual paths for (S)CGI programs, so
PATH_TRANSLATED and PATH_INFO are not set.
.Pp
.It -s
Including this option on the command line, if and only if the -d option has
also been included, causes drood to forward requests for items in the
(S)CGI directory, with a specified filename suffix, to a specified SCGI
Server.  The filesystem entries must exist, but they may be empty files.
.Pp
Multiple instances of this option may be present on the command line.  Each
option value must have form:  <host>|<port>|<suffix>, where <host> is the
name of the host the SCGI server is running on, or its IP address expressed
in either IPv4 or IPv6 presentation format, <port> is the port number the
SCGI server is listening on, and suffix is the filename suffix to associate
with this SCGI server.  Requests for items in the (S)CGI directory whose
filenames are terminated by this sequence of characters, will be forwarded
to the SCGI server at <host> listening on <port>.  The suffix value must
not contain a dot.
.Pp
For example, to forward requests for files matching *.foo to a local server
listening on port 4000, and forward requests for files matching *.bar to a
local server listening on port 5000, one would include
.Bd -literal -offset left
-s '127.0.0.1|4000|foo' -s '127.0.0.1|5000|bar'
.Ed
.Pp
on the drood command line.  Note that it is necessary to quote the option
values to prevent the shell from interpreting them as pipelines.
.Pp
A generic multi-process SCGI server is included in the drood distribution.
The server has its own rc script and manual page (plunger.8), and is used
by the author to test drood's SCGI support.  It allows filter programs to
be used as SCGI servers.
.Pp
.It -p
The argument to the -p option specifies the port to listen on.  This
defaults to 80 if the -p option is not present on the command-line.
.It -i
By default, drood accepts connections on all interfaces it can find capable
of IPv4 or IPv6.  The -i option, when present, overrides this behavior, by
limiting drood to accepting connections from a specified interface only.
The option accepts the IP address of the desired interface as an argument.
The address must be expressed in the presentation format for either IPv4 or
IPv6.
.It -u
.It -g
The -u and the -g options may be used to specify the user and group of
drood server processes.  If not specified, both values default to "nobody".
.It -t
When reading a request header from a client, drood drops the connection if
it cannot read the complete request header within twenty seconds.  The -t
option, if present, specifies a different read timeout value, in seconds.
The value must be in the range 1-60 seconds.
.It -w
When drood is in the process of writing a requested resource to a client,
the connection will be dropped if 60 minutes has elapsed since it started
writing the resource.  This value may be changed with the -w option, and must
be in range of 0-120 minutes.  Setting this value to 0, disables the write
timeout.
.It -q
The -q option, if present, specifies the backlog of client connections
queued by the OS kernel for the server to subsequently service.  This value
defaults to 1000.  Note that the OS kernel actually uses a queue of 1.5
times the size of the specified value.  Connections arriving when the queue
is full are dropped by the kernel.
.It -x
The -x option, if present, prevents drood from becoming a daemon.  It will
then run in the foreground of the terminal where it was started, and may be
stopped with signals (ie., Control-C).  The server also will not write its
pid to /var/run/drood.pid when the -x option is used.
.It -m
The -m option specifies how many instances of drood should be run, and
defaults to the number of CPUs on the host system.  On a four-core machine,
four instances of drood would be run automatically.
.It -l
The -l option turns-on request logging.  If present it must specify the
fully-qualified filename of a file to hold the request log.  The file must
be writeable by the user that drood starts as.  If the file does not exist,
drood will attempt to create it, so the directory in which the file resides
should be writeable by the initial user, as well.
.Pp
Logging is turned-off by default.  Request logging differs from transaction
logging in that requests are logged as they arrive, instead of after they
have been processed.  In particular, this means the response code and the
number of bytes transferred are not recorded in log records, because they
are not known at the time the request is logged.  Drood does not use the
common log file format.  It uses its own, simpler-to-parse format.
.Pp
Each line of the logfile is a complete request record.  The fields of
request records are ordered as follows, with all fields separated by single
| characters:  The date and time of the request, the client IP address, the
request User-Agent header, the request method, the request URL, and
finally, the request Referer (sic) header.  If no User-Agent or Referer
header is present in a request, these items will be the strings, "no
user-agent header", or "no referrer header", respectively.
.Pp
Note that the IP address will be expressed in IPv6 presentation format by
default.  Clients making IPv4 connections will have their IP addresses
written as IPv4-mapped IPv6 addresses, unless the -i option has been
specified with an IPv4 address.  In that case the address with appear in
the presentation format for IPv4.
.Pp
Read the manual for newsyslog.conf(5) to learn how have your logfile
automatically rotated.  An example is shown below.  Adding this line to
/etc/newsyslog.conf will cause the file /var/drood-requests.log to be
turned-over every night at midnight, with the five most recent logfiles
kept on-hand, renamed and compressed with gzip.
.Bd -literal -offset left
/var/log/drood-requests.log 600 5 * T00 Z /var/run/drood.pid
.Ed
.Pp
newsyslog(8) can be scheduled to run with crontab(1,5).  Here is a sample
crontab for root.  Copying these lines to a file and invoking
.Bd -literal -offset left
/usr/bin/crontab <filename>
.Ed
.Pp
will cause crontab to run newsyslog at midnight every night:
.Bd -literal -offset left
SHELL=/bin/sh
PATH=/usr/sbin
HOME=/var/log
# insert your email address below to be notified by mail when newsyslog
# emits output on its stderr.
MAILTO=user@host.domain

0 0 * * *	/usr/sbin/newsyslog.
.Ed
.El
.Sh AUTHORS
.An James Bailie Aq jimmy@mammothcheese.ca
.br
http://www.mammothcheese.ca
