/*
 * Responder Copyright (c) 2010, James Bailie.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *     * The name of James Bailie may not be used to endorse or promote
 * products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <stdarg.h>
#include <string.h>

#define MAX_ENV 40

char *env[ MAX_ENV + 1 ];

void *memory( int size )
{
   void *ptr;

   if (( ptr = malloc( size )) == NULL )
   {
      syslog( LOG_ERR, "malloc(): %m" );
      exit( 1 );
   }

   return ptr;
}

char *str_dup( char *str )
{
   char *ptr;
   int len;

   len = strlen( str );
   ptr = ( char *)memory( len + 1 );
   bcopy( str, ptr, len + 1 );

   return ptr;
}
   
int main( int argc, char **argv )
{
   char buffer[ 256 ], **ptr, *line;
   int i;

   openlog( "responder", LOG_PID, LOG_DAEMON );

   for( i = 0, ptr = env; i <= MAX_ENV; ++i, ++ptr )
      *ptr = NULL;

   for( ; ; )
   {
      for( i = 0, ptr = env; i < MAX_ENV; i += 2 )
      {
         if ( fgets( buffer, sizeof( buffer ), stdin ) == NULL )
            exit(( feof( stdin ) ? 0 : 1 ));

         if ( *buffer == '\n' )
            break;

         if (( line = strchr( buffer, ':' )) == NULL )
            exit( 1 );

         *line++ = '\0';
         *ptr++ = str_dup( buffer );

         line[ strlen( line ) - 1 ] = '\0';
         *ptr++ = str_dup( line );
      }

      if ( i % 2 )
      {
         syslog( LOG_ERR, "CGI variable without value" );
         exit( 1 );
      }

      for( ; ; )
      {
         if ( fgets( buffer, sizeof( buffer ), stdin ) == NULL )
            exit( 1 );

         if ( !strcmp( buffer, "__END__\n" ))
            break;
      }

      fputs( "Content-Type: text/html\n\n"
             "<html>\n"
             "<head><title>Test Responder</title></head>\n"
             "<body>\n"
             "<h1>Test Responder</h1>\n"
             "<table>\n",
             stdout );

      for( ptr = env; *ptr != NULL; ptr += 2 )
         printf( "<tr><td>%s</td><td>%s</td></tr>\n", *ptr, *( ptr + 1 ));

      fputs( "</table>\n</body>\n</html>\n__END__\n", stdout );
      fflush( stdout );

      for( ptr = env; *ptr != NULL; ++ptr )
      {
         free( *ptr );
         *ptr = NULL;
      }
   }
}
